import 'package:apex_socket/apex_socket.dart';
import 'package:flutter/material.dart';

export 'package:socket_io_client/socket_io_client.dart';

class ApexApi extends InheritedApexApi {
  ApexApi(ApexApiConfig config, {Key key, @required Widget child})
      : super(
          config,
          key: key,
          child: child,
        );

  static ApexApi of(BuildContext context) {
    final ApexApi result =
        context.dependOnInheritedWidgetOfExactType<ApexApi>();
    assert(result != null, 'No ApexApi found in context');
    return result;
  }

  static void privateEmit(BuildContext context, String event,
      {String data, bool enc}) {
    final apexApi = of(context);
    apexApi.prEmit(event, data: data, enc: enc);
  }

  static void publicEmit(BuildContext context, String event,
      {String data, bool enc}) {
    final apexApi = of(context);
    apexApi.puEmit(event, data: data, enc: enc);
  }

  static Future<dynamic> privateEmitCompleter(
      BuildContext context, String event,
      {String data, bool enc}) {
    final apexApi = of(context);
    return apexApi.prEmitWithCompleter(event, data, enc);
  }

  static Future<dynamic> publicEmitCompleter(BuildContext context, String event,
      {String data, bool enc}) {
    final apexApi = of(context);
    return apexApi.puEmitWithCompleter(event, data, enc);
  }
}

class InheritedApexApi extends InheritedWidget {
  final ApexApiConfig config;
  final ApexSocket _api;

  InheritedApexApi(
    this.config, {
    Key key,
    @required Widget child,
  })  : _api = ApexSocket(
          config,
          androidKey: config.androidKey,
          iosKey: config.iosKey,
          webKey: config.webKey,
        ),
        super(key: key, child: child) {
    _api.createSocket();
  }

  @override
  bool updateShouldNotify(InheritedApexApi old) => config != old.config;

  void prEmit(String event, {String data, bool enc}) {
    _api.privateEmit(event, data: data, enc: enc);
  }

  void puEmit(String event, {String data, bool enc}) {
    _api.publicEmit(event, data: data, enc: enc);
  }

  void prEmitWithAck(String event, String data,
      {Function(String data) ack, bool enc}) {
    _api.privateEmitWithAck(event, data, ack: ack, enc: enc);
  }

  Future<dynamic> prEmitWithCompleter(String event, String data, [bool enc]) {
    return _api.privateEmitWithAckC(event, data, enc);
  }

  Future<dynamic> puEmitWithCompleter(String event, String data, [bool enc]) {
    return _api.publicEmitWithAckC(event, data, enc);
  }
}

class KeyPair {
  final String secretKey;
  final String publicKey;

  KeyPair(this.secretKey, this.publicKey);
}

class ApexApiConfig {
  final KeyPair iosKey;
  final KeyPair webKey;
  final KeyPair androidKey;

  final String url;
  final int port;
  final OptionBuilder options;

  final String namespace;

  final String privateVersion;
  final String publicVersion;

  ApexApiConfig(
    this.url,
    this.port,
    this.namespace, {
    this.androidKey,
    this.iosKey,
    this.webKey,
    this.privateVersion = '1',
    this.publicVersion = '1',
    @required this.options,
  })  : assert(url != null && Uri.parse(url).isAbsolute,
            '$url must be a valid url.'),
        assert(port == null || (port >= -1 && port <= 65535),
            '$port must be a number between -1 and 65535. or null.'),
        assert(options != null, 'Options must not be null');
}
